
namespace :recycling do

task :help do
puts "Options:
  :refresh_all     kompletně obnov všechno
  :refresh_ips     kompletně obnov IPR 
                   (= :download_ipr, :destroy_ipr, :import_ipr)
  :refresh_osm     kompletně obnov IPR
  :
set_neighbours  vyhledej dvojice co k sobe patri
"
end

task :refresh_all => [ :refresh_ipr, :refresh_osm, :refresh_production_env ]

task :refresh_production_env => [:wipe_cache, :assets_prec] do
  `sudo service nginx restart`
end

task :assets_prec => [ :environment ] do
  system('rake assets:precompile RAILS_ENV=production')
end

task :wipe_cache => [ :environment ] do
  dir = Rails.root.join("public", "cached_pages")
  FileUtils.remove_dir(dir) if Dir.exist?(dir)
  Dir.mkdir(dir)
  imgdir_out = Rails.root.join("public", "cached_pages", 'assets')
  imgdir_in = Rails.root.join('app','assets','images')
  Dir.mkdir(imgdir_out)
  FileUtils.cp(Dir.glob(imgdir_in+'*.png'), imgdir_out)
end

task :refresh_ipr => [ :download_ipr, :destroy_ipr, :import_ipr]
task :destroy_ipr => [:environment] do Point.where(typ: 'ipr').destroy_all end

task :download_ipr  do
  file1out = 'data/ipr_data.json'
  file2out = 'data/ipr_data_items.json'
  if (Time.now-File.mtime(file1out))>60*60  # stahovat jen jednou za hodinu
    `wget -O #{file1out} http://opendata.iprpraha.cz/CUR/ZPK/ZPK_O_Kont_TOstan_b/WGS_84/ZPK_O_Kont_TOstan_b.json`
    `touch #{file1out}`
    `wget -O #{file2out} http://opendata.iprpraha.cz/CUR/ZPK/ZPK_O_Kont_TOitem_b/S_JTSK/ZPK_O_Kont_TOitem_b.json`
    `touch #{file2out}`
  end
end

task :import_ipr => [:environment] do
  puts 'Parsing IPR points datafiles ...'
  ipr_items = JSON.parse(File.read('data/ipr_data_items.json'))['features']
  ipr_points = JSON.parse(File.read('data/ipr_data.json'))['features'].
    select{ |hash| hash['properties']['PRISTUP']=='volně' }.
    map{ |hash| IprPoint.new(hash, ipr_items.select{ |item| 
      item['properties']['STATIONID']==hash['properties']['ID']
    })}
  puts "Creating IPR #{ipr_points.size} points in database ..."
  ipr_points.each do  |ipr_point|
    point = Point.new(ipr_point.def_hash)
    p point
    if !point.save
      raise "Error saving point #{ipr_point.name}"
    end
  end
  at_exit { set_neighbours }
end

class IprPoint # < Location
  def initialize(source_hash, items_array=[]); @src=source_hash; @items=items_array end
  def lat; @src['geometry']['coordinates'][1] end
  def lon; @src['geometry']['coordinates'][0] end
  def tags; @src['properties'] end
  def name; tags['STATIONNAME'] end
  def district; tags['CITYDISTRICT'] end
  def def_hash; {lat: lat, lon: lon, name: name, src: @src.to_json, items_src:@items.to_json, typ:'ipr'} end
end

task :refresh_osm => [ :download_osm, :destroy_osm, :import_osm]
task :destroy_osm => [:environment] do Point.where(typ: 'osm').destroy_all end

task :download_osm  do
  fileout = 'data/osm_data.json'
  if !File.exists?(fileout) or (Time.now-File.mtime(fileout))>60*60  # stahovat jen jednou za hodinu
    district = 'Q1085'
    # district = 'Q3489926'
    `wget -O #{fileout}  'http://overpass-api.de/api/interpreter?data=[out:json];area[wikidata=\"#{district}\"];(node[amenity=\"recycling\"](area););out;'`
  end
end

task :import_osm => [:environment] do
  puts "Parsing OSM JSON file ..."
  osm_points =   JSON.parse(File.read('data/osm_data.json'))['elements'].
    select{ |osm_data| ['container',nil].include? osm_data['tags']['recycling_type'] }.
    map{ |source| OsmPoint.new(source)}
  puts "Creating OSM points in database ..."
  osm_points.each do  |osm_point|
    Point.new(osm_point.def_hash).save
  end
  at_exit { set_neighbours }
end

task :set_neighbours => [:environment] do
  at_exit { set_neighbours }
end

# nastavi near_id u vsech pointu
def set_neighbours
  return if $neighbours_already_set
  puts "Setting neighbours"
  Point.all.each do |point|
    near = point.near
    unless near.empty?
      point.near_id = near.first.id
      point.save
    end
  end
  $neighbours_already_set = true
end

class OsmPoint # < Location
  def initialize(source); @src=source end
  def lat; @src['lat'] end
  def lon; @src['lon'] end
  def tags; @src['tags'] end
  def osmid; @src['id'] end
  def name; tags['name'] || "osm##{osmid}"  end
  def def_hash; {lat: lat, lon: lon, name: name, src: @src.to_json, typ:'osm'} end
end

end # namespace